from schema import Schema, Optional, And, Regex, SchemaError

rpm_schema = Schema({
    Optional('config'): 'wip',
    'content': {
        Optional('build'): {
            'use_source_tito_config': bool,
            'tito_target': And(str, len),
            'push_release_commit': bool,
        },
        Optional('source'): {
            Optional('alias'): And(str, len),
            Optional('git'): {
                'branch': {
                    'fallback': And(str, len),
                    'target': And(str, len),
                },
                'url': And(str, len),
            },
            'specfile': Regex(r'.+\.spec$'),
        },
    },
    Optional('enabled_repos'): [
        And(str, len),
    ],
    'name': And(str, len),
    'owners': [
        And(str, len)
    ],
})


def validate(_, data):
    try:
        rpm_schema.validate(data)
    except SchemaError as err:
        return '{}'.format(err)



import os
from . import support


def validate(file, data, group_cfg):
    endpoint = get_cgit_endpoint(group_cfg)
    namespace = get_namespace(data, support.get_artifact_type(file))
    repository = get_repository_name(file)

    url = '{}/{}/{}'.format(endpoint, namespace, repository)

    if not support.resource_is_reacheable(endpoint):
        return (url, ('This validation must run from a network '
                      'with access to {}'.format(endpoint)))

    if not support.resource_exists(url):
        return (url, ('Corresponding DistGit repo was not found.\n'
                      "If you didn't request a DistGit repo yet, "
                      'please check https://mojo.redhat.com/docs/DOC-1168290\n'
                      'But if you already obtained one, make sure its name '
                      'matches the YAML filename'))

    branch = get_distgit_branch(data, group_cfg)
    if not branch_exists(branch, url):
        return (url, ('Branch {} not found on DistGit'.format(branch)))

    return (url, None)


def get_cgit_endpoint(group_cfg):
    return group_cfg['urls']['cgit']


def get_namespace(data, artifact_type):
    if 'distgit' in data and 'namespace' in data['distgit']:
        return data['distgit']['namespace']

    return {'image': 'containers', 'rpm': 'rpms'}.get(artifact_type, '???')


def get_repository_name(file):
    return os.path.basename(file).split('.')[0]


def get_distgit_branch(data, group_cfg):
    if 'distgit' in data and 'branch' in data['distgit']:
        return data['distgit']['branch']

    return (group_cfg['branch']
            .replace('{MAJOR}', str(group_cfg['vars']['MAJOR']))
            .replace('{MINOR}', str(group_cfg['vars']['MINOR'])))


def branch_exists(branch, url):
    return support.resource_exists('{}/log?h={}'.format(url, branch))



from validator import support
from schema import Schema, Optional, Use, And, Or, SchemaError


def image_schema(file):
    valid_arches = [
        'x86_64',
    ]

    valid_modification_actions = [
        'command',
        'replace',
    ]

    valid_modification_commands = [
        'update-console-sources',
    ]

    valid_distgit_namespaces = [
        'apbs',
        'containers',
        'rpms',
    ]

    valid_streams = support.get_valid_streams_for(file)
    valid_member_references = support.get_valid_member_references_for(file)

    return Schema({
        Optional('arches'): [Or(*valid_arches)],
        Optional('base_only'): True,
        Optional('config'): 'wip',
        Optional('container_yaml'): {
            'go': {
                'modules': [
                    {
                        'module': And(str, len),
                        Optional('path'): str,
                    },
                ],
            },
        },
        Optional('content'): {
            'source': {
                Optional('alias'): And(str, len),
                Optional('dockerfile'): And(str, len),
                Optional('git'): {
                    'branch': {
                        Optional('fallback'): And(str, len),
                        'target': And(str, len),
                    },
                    'url': And(Use(str), len),
                },
                Optional('modifications'): [{
                    'action': Or(*valid_modification_actions),
                    Optional('command'): [
                        Or(*valid_modification_commands),
                    ],
                    Optional('match'): And(str, len),
                    Optional('replacement'): And(str, len),
                }],
                Optional('path'): str,
            },
        },
        Optional('dependents'): [
            And(str, len)
        ],
        Optional('distgit'): {
            Optional('namespace'): Or(*valid_distgit_namespaces),
            Optional('component'): And(str, len),
            Optional('branch'): And(str, len),
        },
        Optional('enabled_repos'): [
            And(str, len),
        ],
        'from': {
            Optional('builder'): [
                {
                    Optional('stream'): Or(*valid_streams),
                    Optional('member'): Or(*valid_member_references),
                    Optional('image'): And(str, len),
                },
            ],
            Optional('image'): And(str, len),
            Optional('stream'): Or(*valid_streams),
            Optional('member'): Or(*valid_member_references),
        },
        Optional('labels'): {
            Optional('License'): And(str, len),
            Optional('io.k8s.description'): And(str, len),
            Optional('io.k8s.display-name'): And(str, len),
            Optional('io.openshift.tags'): And(str, len),
            Optional('vendor'): And(str, len),
        },
        Optional('mode'): And(str, len),
        'name': And(str, len),
        Optional('odcs'): {
            'packages': {
                'exclude': [
                    And(str, len),
                ],
                'mode': 'auto',
            },
        },
        Optional('no_oit_comments'): bool,
        Optional('owners'): [
            And(str, len),
        ],
        Optional('push'): {
            'repos': [
                And(str, len),
            ],
        },
        Optional('required'): bool,
        Optional('update-csv'): {
            'manifests-dir': And(str, len),
            'registry': And(str, len),
        },
        Optional('wait_for'): And(str, len),
    })


def validate(file, data):
    try:
        image_schema(file).validate(data)
    except SchemaError as err:
        return '{}'.format(err)



import setuptools

setuptools.setup(
    name='rh-ocp-build-data-validator',
    author='AOS ART Team',
    author_email='aos-team-art@redhat.com',
    version='0.0.4',
    description='Validation of ocp-build-data Image & RPM declarations',
    long_description=open('README.rst').read(),
    url='https://gitlab.cee.redhat.com/openshift-art/tools/ocp-build-data-validator',  # noqa: E501
    license='Red Hat Internal',
    packages=['validator', 'validator.schema'],
    entry_points={'console_scripts': [
        'validate-ocp-build-data = validator.__main__:main'
    ]},
    include_package_data=True,
    install_requires=['pyyaml', 'schema', 'requests'])
